import { IsSupported } from "../IsSupported/IsSupported.svelte.js";
/**
 * Reactive access to the browser's [Geolocation API](https://developer.mozilla.org/en-US/docs/Web/API/Geolocation_API).
 *
 * @see https://runed.dev/docs/utilities/use-geolocation
 */
export function useGeolocation(options = {}) {
    const { enableHighAccuracy = true, maximumAge = 30000, timeout = 27000, immediate = true, } = options;
    const isSupported = new IsSupported(() => navigator && "geolocation" in navigator);
    let locatedAt = $state(null);
    let error = $state.raw(null);
    let coords = $state({
        accuracy: 0,
        latitude: Number.POSITIVE_INFINITY,
        longitude: Number.POSITIVE_INFINITY,
        altitude: null,
        altitudeAccuracy: null,
        heading: null,
        speed: null,
    });
    let isPaused = $state(false);
    function updatePosition(position) {
        locatedAt = position.timestamp;
        coords.accuracy = position.coords.accuracy;
        coords.altitude = position.coords.altitude;
        coords.altitudeAccuracy = position.coords.altitudeAccuracy;
        coords.heading = position.coords.heading;
        coords.latitude = position.coords.latitude;
        coords.longitude = position.coords.longitude;
        coords.speed = position.coords.speed;
    }
    let watcher;
    function resume() {
        if (!isSupported.current)
            return;
        watcher = navigator.geolocation.watchPosition(updatePosition, (err) => (error = err), {
            enableHighAccuracy,
            maximumAge,
            timeout,
        });
        isPaused = false;
    }
    function pause() {
        if (watcher && navigator)
            navigator.geolocation.clearWatch(watcher);
        isPaused = true;
    }
    $effect(() => {
        if (immediate)
            resume();
        return () => pause();
    });
    return {
        get isSupported() {
            return isSupported.current;
        },
        get coords() {
            return coords;
        },
        get locatedAt() {
            return locatedAt;
        },
        get error() {
            return error;
        },
        get isPaused() {
            return isPaused;
        },
        resume,
        pause,
    };
}
